@@ -29,6 +29,7 @@
 #include <unistd.h>
 
 #include <glib/gstdio.h>
+#include <glib/gi18n.h>
 
 #include "seahorse-util.h"
 #include "seahorse-libdialogs.h"
@@ -112,7 +113,8 @@ seahorse_gpgme_key_op_generate (SeahorseGpgmeSource *psrc, const gchar *name,
         if (length < DSA_MIN || length > DSA_MAX)
             *err = GPG_E (GPG_ERR_INV_VALUE);
         break;
-    case RSA_RSA: RSA_SIGN:
+    case RSA_RSA:
+    case RSA_SIGN:
         if (length < RSA_MIN || length > LENGTH_MAX)
             *err = GPG_E (GPG_ERR_INV_VALUE);
         break;
@@ -294,6 +296,10 @@ edit_gpgme_key (gpgme_ctx_t ctx, gpgme_key_t key, SeahorseEditParm *parms)
     
 	/* do edit callback, release data */
 	gerr = gpgme_op_edit (ctx, key, seahorse_gpgme_key_op_edit, parms, out);
+
+	if (gpgme_err_code (gerr) == GPG_ERR_BAD_PASSPHRASE) {
+		seahorse_util_show_error(NULL, _("Wrong password"), _("This was the third time you entered a wrong password. Please try again."));
+	}
 	
 	seahorse_gpgme_data_release (out);
     
@@ -532,11 +538,7 @@ sign_process (gpgme_key_t signed_key, gpgme_key_t signing_key, guint sign_index,
 	g_free (parms);
 	 
 	gpgme_release (ctx);
-	
-	/* If it was already signed then it's not an error */
-	if (gpgme_err_code (gerr) == GPG_ERR_EALREADY)
-		gerr = GPG_OK;
-	    
+
 	return gerr;
 }
 
@@ -643,7 +645,7 @@ edit_pass_transit (guint current_state, gpgme_status_code_t status,
         
         /* If all tries for passphrase were wrong, we get here */
         else if (status == GPGME_STATUS_GET_LINE && g_str_equal (args, PROMPT)) {
-            *err = GPG_E (GPG_ERR_CANCELED);
+            *err = GPG_E (GPG_ERR_BAD_PASSPHRASE);
             next_state = PASS_ERROR;
         
         /* No idea how we got here ... */
@@ -819,12 +821,12 @@ edit_trust_transit (guint current_state, gpgme_status_code_t status,
 
 /**
  * seahorse_gpgme_key_op_set_trust:
- * @skey: #SeahorseKey whose trust will be changed
- * @trust: New trust value that must be at least #SEAHORSE_VALIDITY_UNKNOWN.
- * If @skey is a #SeahorseKeyPair, then @trust cannot be #SEAHORSE_VALIDITY_UNKNOWN.
- * If @skey is not a #SeahorseKeyPair, then @trust cannot be #SEAHORSE_VALIDITY_ULTIMATE.
+ * @pkey: #SeahorseGpgmeKey whose trust will be changed
+ * @trust: New trust value that must be at least #SEAHORSE_VALIDITY_NEVER.
+ * If @pkey is a #SeahorseKeyPair, then @trust cannot be #SEAHORSE_VALIDITY_UNKNOWN.
+ * If @pkey is not a #SeahorseKeyPair, then @trust cannot be #SEAHORSE_VALIDITY_ULTIMATE.
  *
- * Tries to change the owner trust of @skey to @trust.
+ * Tries to change the owner trust of @pkey to @trust.
  *
  * Returns: Error value
  **/
@@ -837,7 +839,7 @@ seahorse_gpgme_key_op_set_trust (SeahorseGpgmeKey *pkey, SeahorseValidity trust)
 	DEBUG_OPERATION(("[GPGME_KEY_OP] set_trust: trust = %i",trust));
 	
 	g_return_val_if_fail (SEAHORSE_IS_GPGME_KEY (pkey), GPG_E (GPG_ERR_WRONG_KEY_USAGE));
-	g_return_val_if_fail (trust >= GPGME_VALIDITY_UNKNOWN, GPG_E (GPG_ERR_INV_VALUE));
+	g_return_val_if_fail (trust >= SEAHORSE_VALIDITY_NEVER, GPG_E (GPG_ERR_INV_VALUE));
 	g_return_val_if_fail (seahorse_gpgme_key_get_trust (pkey) != trust, GPG_E (GPG_ERR_INV_VALUE));
 	
 	if (seahorse_object_get_usage (SEAHORSE_OBJECT (pkey)) == SEAHORSE_USAGE_PRIVATE_KEY)
@@ -846,12 +848,12 @@ seahorse_gpgme_key_op_set_trust (SeahorseGpgmeKey *pkey, SeahorseValidity trust)
 		g_return_val_if_fail (trust != SEAHORSE_VALIDITY_ULTIMATE, GPG_E (GPG_ERR_INV_VALUE));
 	
 	switch (trust) {
-        case SEAHORSE_VALIDITY_UNKNOWN:
-            menu_choice = GPG_UNKNOWN;
-            break;
         case SEAHORSE_VALIDITY_NEVER:
             menu_choice = GPG_NEVER;
             break;
+        case SEAHORSE_VALIDITY_UNKNOWN:
+            menu_choice = GPG_UNKNOWN;
+            break;
         case SEAHORSE_VALIDITY_MARGINAL:
             menu_choice = GPG_MARGINAL;
             break;
@@ -1392,7 +1394,7 @@ typedef enum {
 } AddKeyState;
 
 typedef struct {
-	SeahorseKeyEncType  type;
+	guint               type;
 	guint               length;
 	time_t              expires;
 } SubkeyParm;
@@ -1498,19 +1500,31 @@ seahorse_gpgme_key_op_add_subkey (SeahorseGpgmeKey *pkey, const SeahorseKeyEncTy
 {
 	SeahorseEditParm *parms;
 	SubkeyParm *key_parm;
+	guint real_type;
+	SeahorseKeyTypeTable table;
+	gpgme_error_t gerr;
 	
     g_return_val_if_fail (SEAHORSE_IS_GPGME_KEY (pkey), GPG_E (GPG_ERR_WRONG_KEY_USAGE));    
     g_return_val_if_fail (seahorse_object_get_usage (SEAHORSE_OBJECT (pkey)) == SEAHORSE_USAGE_PRIVATE_KEY, GPG_E (GPG_ERR_WRONG_KEY_USAGE));
 	
+	gerr = seahorse_gpgme_get_keytype_table (&table);
+	g_return_val_if_fail (GPG_IS_OK (gerr), gerr);
+	
 	/* Check length range & type */
 	switch (type) {
 		case DSA:
+			real_type = table->dsa_sign;
 			g_return_val_if_fail (length >= DSA_MIN && length <= DSA_MAX, GPG_E (GPG_ERR_INV_VALUE));
 			break;
 		case ELGAMAL:
+			real_type = table->elgamal_enc;
 			g_return_val_if_fail (length >= ELGAMAL_MIN && length <= LENGTH_MAX, GPG_E (GPG_ERR_INV_VALUE));
 			break;
 		case RSA_SIGN: case RSA_ENCRYPT:
+			if (type == RSA_SIGN)
+				real_type = table->rsa_sign;
+			else
+				real_type = table->rsa_enc;
 			g_return_val_if_fail (length >= RSA_MIN && length <= LENGTH_MAX, GPG_E (GPG_ERR_INV_VALUE));
 			break;
 		default:
@@ -1519,7 +1533,7 @@ seahorse_gpgme_key_op_add_subkey (SeahorseGpgmeKey *pkey, const SeahorseKeyEncTy
 	}
 	
 	key_parm = g_new (SubkeyParm, 1);
-	key_parm->type = type;
+	key_parm->type = real_type;
 	key_parm->length = length;
 	key_parm->expires = expires;
 	
@@ -1689,11 +1703,10 @@ rev_subkey_action (guint state, gpointer data, int fd)
             PRINTF ((fd, "%d", parm->reason));
 			break;
 		case REV_SUBKEY_DESCRIPTION:
-            PRINTF ((fd, "%s\n", parm->description));
+            PRINTF ((fd, "%s", parm->description));
 			break;
-		/* Need empty line */
 		case REV_SUBKEY_ENDDESC:
-            PRINT ((fd, "\n"));
+            /* Need empty line, which is written at the end */
 			break;
 		case REV_SUBKEY_QUIT:
             PRINT ((fd, QUIT));
@@ -1712,7 +1725,7 @@ rev_subkey_transit (guint current_state, gpgme_status_code_t status,
 		    const gchar *args, gpointer data, gpgme_error_t *err)
 {
 	guint next_state;
-	
+
 	switch (current_state) {
 		/* start, select key */
 		case REV_SUBKEY_START:
@@ -1776,7 +1789,12 @@ rev_subkey_transit (guint current_state, gpgme_status_code_t status,
 			break;
 		/* ended description, confirm */
 		case REV_SUBKEY_ENDDESC:
-			next_state = REV_SUBKEY_CONFIRM;
+			if (status == GPGME_STATUS_GET_BOOL && g_str_equal (args, "ask_revocation_reason.okay"))
+				next_state = REV_SUBKEY_CONFIRM;
+			else {
+                *err = GPG_E (GPG_ERR_GENERAL);
+                g_return_val_if_reached (REV_SUBKEY_ERROR);
+			}
 			break;
 		/* quit, confirm(=save) */
 		case REV_SUBKEY_QUIT:
